package hdm.mi.growbros.service;

import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.repositories.PlantRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;
import java.util.stream.IntStream;

import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SpringBootTest
@ActiveProfiles("test")
class PlantsServiceTest {
    @MockBean
    private PlantRepository plantRepository;

    @Autowired
    private PlantsService plantsService;

    @Test
    void verify_repositoryCalled_withCorrectPageable() {
        final PageRequest correctPageable = PageRequest.of(0, 10);
        PageImpl<Plant> page = new PageImpl<>(List.of());
        when(plantRepository.findAll(any(PageRequest.class)))
                .thenReturn(page);

        plantsService.getPlants(0, 10);

        verify(plantRepository).findAll(correctPageable);
    }

    @Test
    void getPlants_withPageEmpty() {
        final PageRequest pageable = PageRequest.of(0, 10);
        PageImpl<Plant> page = new PageImpl<>(List.of());
        final CustomPageDto<Plant> expected = new CustomPageDto<>(0, 0, page.getContent());
        when(plantRepository.findAll(pageable))
                .thenReturn(page);

        CustomPageDto<Plant> actual = plantsService.getPlants(0, 10);

        assertAll(
                () -> assertEquals(expected.currentPage(), actual.currentPage()),
                () -> assertEquals(0, actual.pageSize()),
                () -> assertEquals(expected.content(), actual.content())
        );
    }

    @Test
    void getPlants_correctPageSize() {
        int plantAmount = 5;
        var plants = getMockPlants(plantAmount);

        final PageRequest pageable = PageRequest.of(0, 10);
        PageImpl<Plant> page = new PageImpl<>(plants);
        final CustomPageDto<Plant> expected = new CustomPageDto<>(0, plantAmount, page.getContent());
        when(plantRepository.findAll(pageable))
                .thenReturn(page);

        CustomPageDto<Plant> actual = plantsService.getPlants(0, 10);

        assertAll(
                () -> assertEquals(expected.currentPage(), actual.currentPage()),
                () -> assertEquals(plantAmount, actual.pageSize()),
                () -> assertEquals(expected.content(), actual.content())
        );
    }

    @Test
    void getPlants_pageNumberNonNegative() {
        int pageSize = 10;
        final PageRequest correctPageRequest = PageRequest.of(0, pageSize);
        when(plantRepository.findAll(any(PageRequest.class)))
                .thenReturn(new PageImpl<>(List.of()));

        plantsService.getPlants(-1, pageSize);

        // calling the service with -1 as page number, and verifying that the repository is called
        // with the correct page number
        verify(plantRepository).findAll(correctPageRequest);
    }

    @Test
    void pageSize_cappedAt_100() {
        int pageNumber = 0;
        final PageRequest correctPageRequest = PageRequest.of(pageNumber, 100);
        when(plantRepository.findAll(any(PageRequest.class)))
                .thenReturn(new PageImpl<>(List.of()));

        plantsService.getPlants(pageNumber, 1000);

        verify(plantRepository).findAll(correctPageRequest);
    }

    private List<Plant> getMockPlants(int amount) {
        return IntStream
                .range(0, amount)
                .mapToObj(index -> Plant.builder().name(String.format("My generated plant %d", index)).build())
                .toList();
    }
}
